import {Component, Input} from '@angular/core';
import {HelperFunctions} from "../../utils/HelperFunctions.class";
import {HostedByCollectedFrom} from "../../utils/result-preview/result-preview";
import {properties} from "../../../../environments/environment";

@Component({
  selector: 'availableOn',
  template: `
    <div class="download-from uk-height-max-large uk-overflow-auto uk-margin-small-bottom">
      <div *ngFor="let available of availableOn.slice(0, showNum) let i=index"
           class="uk-flex uk-flex-top"
           [title]="available.bestAccessMode ? available.bestAccessMode : 'Not available'">
        <span class="uk-margin-small-right">
          <img [src]="available.icon" loading="lazy" [alt]="available.bestAccessMode" style="width:20px; height:20px">
        </span>
        <div class="uk-width-expand uk-padding-small uk-padding-remove-left uk-padding-remove-vertical">
          <span *ngIf="available.downloadUrl.length > 1" class="title">
            <span>{{available.downloadName}}</span>
            <a *ngFor="let url of available.downloadUrl; let i=index;"
               [href]="url" target="_blank">
               [{{(i + 1) | number}}]
            </a>
          </span>
          <a *ngIf="available.downloadUrl.length === 1" [href]="available.downloadUrl[0]" target="_blank"
             class="title">
            {{available.downloadName}}
            <span class="custom-external custom-icon space"></span>
          </a>
          <span *ngIf="!available.downloadUrl || available.downloadUrl.length === 0" class="title">
            {{available.downloadName}}
          </span>
          <div *ngIf="removeUnknown(available.type) || available.year">
            <span *ngIf="removeUnknown(available.type)" class="uk-text-capitalize">{{available.type}}</span>
            <span *ngIf="removeUnknown(available.type) && available.year"> . </span>
            <span *ngIf="available.year">{{available.year}}</span>
          </div>
          <div *ngIf="available.collectedName">
            <span>Provider: </span>
            <a *ngIf="available.collectedId" [routerLink]="dataProviderUrl"
               [queryParams]="{datasourceId: available.collectedId}">
              {{available.collectedName}}
            </a>
          </div>
        </div>
      </div>
    </div>
    <div *ngIf="showNum > threshold" class="uk-margin-bottom">
      <a (click)="showNum = threshold;" class="uk-flex uk-flex-middle uk-flex-center">
        <span>View less</span>
        <span class="space uk-icon">
          <svg width="20" height="20" viewBox="0 0 20 20" xmlns="http://www.w3.org/2000/svg" data-svg="chevron-up">
            <polyline fill="none" stroke="#000" stroke-width="1.03" points="4 13 10 7 16 13"></polyline>
          </svg>
        </span>
      </a>
    </div>
    <div *ngIf="showNum == threshold && availableOn && availableOn.length > 5" class="uk-margin-bottom">
      <a (click)="showNum = availableOn.length;" class="uk-flex uk-flex-middle uk-flex-center">
        <span>View more</span>
        <span class="space uk-icon">
          <svg width="20" height="20" viewBox="0 0 20 20" xmlns="http://www.w3.org/2000/svg" data-svg="chevron-down">
            <polyline fill="none" stroke="#000" stroke-width="1.03" points="16 7 10 13 4 7"></polyline>
          </svg>
        </span>
      </a>
    </div>
  `
})

export class AvailableOnComponent {
  @Input() availableOn: HostedByCollectedFrom[];
  public threshold: number = 4;
  public showNum: number = 4;
  public dataProviderUrl = properties.searchLinkToDataProvider.split('?')[0];
  
  constructor() {
  }
  
  ngOnInit() {}
  
  public removeUnknown(value: string): string {
    if (value.toLowerCase() === 'unknown') {
      return null;
    }
    return value;
  }
}
